import sys
import traceback as tb

import rapidfuzz as rf
from pydantic import validator

from ipc.utility.helper import build_validation_schema_sample_level, split_ontological_fields_by_type
from ipc.validator.settings import FIELD_MAPPING, VALID_NAMES, VALID_NAMES_LOWERED

ontological_fields = ['curated_disease',
                      'curated_cell_type',
                      'curated_cell_line',
                      'curated_drug',
                      'curated_tissue']

"""---Validator Functions---"""


def check_for_nan_strings(cls, val):
    """
    Validator to check for 'nan' strings
    Args:
        cls:
        val:

    Returns:

    """
    if val == 'nan':
        raise ValueError(f'Value Check | Invalid Value: "{val}"')
    return val


def check_for_default_values(cls, val, field):
    """
        Check for a single None value
        Args:
            val:
            field:

        Returns:

        """
    if isinstance(val, list):
        if len(val) > 1:
            field_to_check = field.name
            # field.name at times comes with a leading underscore
            field_to_check = field_to_check.strip('_')
            if field_to_check == 'curated_disease':
                if 'Normal' in val:
                    raise ValueError(
                        f'Value Check | Contains one or more default value "Normal" | Erroneous Value: "{val}"')
            elif 'none' in val or 'Normal' in val:
                if len(val) > 1:
                    raise ValueError(f'Value Check | Contains one or more default values. | Erroneous Value: "{val}"')
        else:
            if 'NONE' in val or 'nan' in val:
                raise ValueError(f'Value Check | Contains invalid value. | Erroneous Value: "{val}"')
    return val


def check_valid_names(cls, val, field):
    """
        Check that the values of the above field  match the valid values as in the ontology knowledge
        base
        Args:
            val:
            field:

        Returns:
            val
        """
    field_to_check = field.name
    # field.name at times comes with a leading underscore
    field_to_check = field_to_check.strip('_')

    if field_to_check == 'curated_cell_line':
        # Specifically for kw_cell_line, valid names are stores separately.
        valid = VALID_NAMES[FIELD_MAPPING[field_to_check]]['valid_cell_line_names']
        valid_lowered_stripped = VALID_NAMES_LOWERED[FIELD_MAPPING[field_to_check]]['valid_cell_line_names']
    else:
        valid = VALID_NAMES[FIELD_MAPPING[field_to_check]]
        valid_lowered_stripped = VALID_NAMES_LOWERED[FIELD_MAPPING[field_to_check]]
    if val != 'none' and val != 'Normal':
        if val not in valid:
            if val.lower().strip() in valid_lowered_stripped:
                possible_match = rf.process.extract(val, valid, scorer=rf.fuzz.ratio, limit=1)[0][0]
                raise ValueError(f'Ontological Validity Check | Valid Value BUT with case/whitespaces mismatch '
                                 f'| Erroneous Value: "{val}" | Possible match: "{possible_match}"')
            else:
                raise ValueError(f'Ontological Validity Check | Invalid Value | Erroneous Value: "{val}"')
    return val


"""---Function to dynamically build the schema_constructors given a repo---"""


def build_schema_for_repo(desc_df):
    """
    given a repo, build two schema_constructors, one with just the schema, the other with validators as well
    Args:
        desc_df:

    Returns:

    """
    try:
        schema = build_validation_schema_sample_level(desc_df)
        all_curated_columns = desc_df.loc[desc_df['column_name'].str.startswith('curated'), 'column_name'].to_list()
        ontological_fields_as_str, ontological_fields_as_list = split_ontological_fields_by_type(ontological_fields,
                                                                                                 desc_df)

        validators = {'check_for_nan_strings': validator(*all_curated_columns,
                                                         allow_reuse=True,
                                                         check_fields=False)(check_for_nan_strings),
                      'check_for_default_values': validator(*(ontological_fields_as_list + ontological_fields_as_str),
                                                            allow_reuse=True,
                                                            check_fields=False)(check_for_default_values)}
        if ontological_fields_as_list:
            validators['check_valid_names_in_lists'] = validator(*ontological_fields_as_list,
                                                                 allow_reuse=True,
                                                                 each_item=True,
                                                                 check_fields=False)(check_valid_names)
        if ontological_fields_as_str:
            validators['check_valid_names_in_strings'] = validator(*ontological_fields_as_str,
                                                                   allow_reuse=True,
                                                                   check_fields=False)(check_valid_names)
        schema_with_validator = build_validation_schema_sample_level(desc_df, validators=validators)
        return schema, schema_with_validator
    except Exception as exc:
        print(tb.format_exc())
        exception_type, exception_object, exception_traceback = sys.exc_info()
        filename = exception_traceback.tb_frame.f_code.co_filename
        line_number = exception_traceback.tb_lineno
        print("Exception type: ", exception_type)
        print("File name: ", filename)
        print("Line number: ", line_number)
        return None, None
